@@ -26,7 +26,8 @@ plot.igraph <- function(x, layout=layout.random, layout.par=list(),
                        vertex.color="SkyBlue2", vertex.size=15,
                        edge.color="darkgrey", edge.width=1,
                        edge.labels=NA, 
-                       vertex.frame.color="black",
+                       vertex.frame.color="black", 
+                       margin=0, loop.angle=0,
                        # SPECIFIC: #####################################
                        axes=FALSE, xlab="", ylab="",
                        xlim=c(-1,1), ylim=c(-1,1),
@@ -49,6 +50,8 @@ plot.igraph <- function(x, layout=layout.random, layout.par=list(),
   edge.labels <- i.get.edge.labels(graph, edge.labels)
 
   # create the plot
+  xlim <- c(xlim[1]-margin, xlim[2]+margin)
+  ylim <- c(ylim[1]-margin, ylim[2]+margin)
   plot(0, 0, type="n", xlab=xlab, ylab=ylab, asp=1, xlim=xlim, ylim=ylim,
        axes=axes, ...)
 
@@ -56,20 +59,27 @@ plot.igraph <- function(x, layout=layout.random, layout.par=list(),
   layout <- i.layout.norm(layout, -1, 1, -1, 1)
   
   # add the edges
-  # TODO: loops
-  x0 <- layout[,1][get.edgelist(graph)[,1]+1]
-  y0 <- layout[,2][get.edgelist(graph)[,1]+1]
-  x1 <- layout[,1][get.edgelist(graph)[,2]+1]
-  y1 <- layout[,2][get.edgelist(graph)[,2]+1]
+  el <- get.edgelist(graph, names=FALSE)
+  loops.v <- el[,1] [ el[,1] == el[,2] ] + 1
+  loops.e <- which(el[,1] == el[,2])
+  loop.labels <- edge.labels[el[,1] == el[,2]]
+  edge.labels <- edge.labels[el[,1] != el[,2]]
+  el <- el[el[,1] != el[,2],]
+  dim(el) <- c(length(el)/2, 2)
+  x0 <- layout[,1][el[,1]+1]
+  y0 <- layout[,2][el[,1]+1]
+  x1 <- layout[,1][el[,2]+1]
+  y1 <- layout[,2][el[,2]+1]
 
   # we do this for undirected graphs also because some
   # graphics drivers do not handle 'depth' properly (or at all)
   if (length(vertex.size)!=1) {
-    vsize.from <- vertex.size[get.edgelist(graph)[,1]+1]
-    vsize.to   <- vertex.size[get.edgelist(graph)[,2]+1]
+    vsize.from <- vertex.size[el[,1]+1]
+    vsize.to   <- vertex.size[el[,2]+1]
   } else {
     vsize.from <- vsize.to <- vertex.size
   }
+  rm (el)
   phi <- atan2(y1-y0, x1-x0)
   r <- sqrt( (x1-x0)^2 + (y1-y0)^2 )
   x1 <- x0 + (r-vsize.to)*cos(phi)
@@ -77,16 +87,93 @@ plot.igraph <- function(x, layout=layout.random, layout.par=list(),
   x0 <- x0 + vsize.from*cos(phi)
   y0 <- y0 + vsize.from*sin(phi)
   
+  # add the loop edges
+  if (length(loops.e) > 0) {
+    ec <- edge.color
+    if (length(ec)>1) { ec <- ec[loops.e] }
+
+    point.on.cubic.bezier <- function(cp, t) {
+
+      c <- 3 * (cp[2,] - cp[1,])
+      b <- 3 * (cp[3,] - cp[2,]) - c
+      a <- cp[4,] - cp[1,] - c - b
+      
+      t2 <- t*t;
+      t3 <- t*t*t
+      
+      a*t3 + b*t2 + c*t + cp[1,]
+    }
+
+    compute.bezier <- function(cp, points) {
+      dt <- seq(0, 1, by=1/(points-1))
+      sapply(dt, function(t) point.on.cubic.bezier(cp, t))
+    }
+    
+    plot.bezier <- function(cp, points, color, width, arrows) {
+      p <- compute.bezier( cp, points )
+      polygon(p[1,], p[2,], border=color, lwd=width)
+      if (arrows) {
+        arrows(p[1,ncol(p)-1], p[2,ncol(p)-1], p[1,ncol(p)], p[2,ncol(p)],
+               length=.2, angle=20, col=color)
+      }
+    }
+    
+    loop <- function(x0, y0, cx=x0, cy=y0, color, angle=0, label=NA,
+                     width=1, arrows=FALSE) {
+
+      rad <- angle/180*pi
+      center <- c(cx,cy)
+      cp <- matrix( c(x0,y0, x0+.4,y0+.2, x0+.4,y0-.2, x0,y0),
+                   nc=2, byrow=TRUE)
+      phi <- atan2(cp[,2]-center[2], cp[,1]-center[1])
+      r <- sqrt((cp[,1]-center[1])**2 + (cp[,2]-center[2])**2)
+      
+      phi <- phi + rad
+      
+      cp[,1] <- cx+r*cos(phi)
+      cp[,2] <- cy+r*sin(phi)
+
+      plot.bezier(cp, 50, color, width, arrows=arrows)
+
+      if (!is.na(label)) {
+        lx <- x0+.3
+        ly <- y0
+        phi <- atan2(ly-center[2], lx-center[1])
+        r <- sqrt((lx-center[1])**2 + (ly-center[2])**2)
+
+        phi <- phi + rad
+
+        lx <- cx+r*cos(phi)
+        ly <- cy+r*sin(phi)
+
+        text(lx, ly, label, col=label.color)
+      }
+    }
+
+    ec <- edge.color
+    if (length(ec)>1) { ec <- ec[loops.e] }
+    vs <- vertex.size
+    if (length(vertex.size)>1) { vs <- vs[loops.e] }
+    ew <- edge.width
+    if (length(edge.width)>1) { ew <- ew[loops.el] }
+    xx0 <- layout[loops.v,1] + cos(loop.angle[loops.e]/180*pi) * vs
+    yy0 <- layout[loops.v,2] + sin(loop.angle[loops.e]/180*pi) * vs
+    mapply(loop, xx0, yy0,
+           color=ec, angle=loop.angle[loops.e], label=loop.labels,
+           width=ew, MoreArgs=list(arrows=is.directed(graph)))
+
+  }
+  
   arrow.code <- ifelse(is.directed(graph), 2, 0)
-  if (length(x0) != 0) {
+  if (length(x0) != 0) {    
     arrows(x0, y0, x1, y1, angle=20, length=0.2, code=arrow.code,
            col=edge.color, lwd=edge.width)
     x <- (x0+x1)/2
     y <- (y0+y1)/2
+    if (!is.null(label.font)) par(family=label.font)
     text(x, y, labels=edge.labels, col=label.color)
   }
-  # add the edge labels 
-
+  
   rm(x0, y0, x1, y1)
   
   # add the vertices
@@ -148,12 +235,13 @@ rglplot.igraph <- function(x, layout=layout.random, layout.par=list(),
   
   # add the edges
   # TODO: loops
-  x0 <- layout[,1][get.edgelist(graph)[,1]+1]
-  y0 <- layout[,2][get.edgelist(graph)[,1]+1]
-  z0 <- layout[,3][get.edgelist(graph)[,1]+1]
-  x1 <- layout[,1][get.edgelist(graph)[,2]+1]
-  y1 <- layout[,2][get.edgelist(graph)[,2]+1]
-  z1 <- layout[,3][get.edgelist(graph)[,2]+1]
+  el <- get.edgelist(graph, names=FALSE)
+  x0 <- layout[,1][el[,1]+1]
+  y0 <- layout[,2][el[,1]+1]
+  z0 <- layout[,3][el[,1]+1]
+  x1 <- layout[,1][el[,2]+1]
+  y1 <- layout[,2][el[,2]+1]
+  z1 <- layout[,3][el[,2]+1]
 
   # we do this for undirected graphs also because some
   # graphics drivers do not handle 'depth' properly (or at all)
@@ -163,7 +251,8 @@ rglplot.igraph <- function(x, layout=layout.random, layout.par=list(),
   } else {
     vsize.from <- vsize.to <- vertex.size
   }
-
+  rm(el)
+  
   rgl.lines(as.numeric(t(matrix( c(x0,x1), nc=2))),
             as.numeric(t(matrix( c(y0,y1), nc=2))),
             as.numeric(t(matrix( c(z0,z1), nc=2))),
